@@ -40,6 +40,9 @@ namespace sharp {
   {
     boost::filesystem::path p(dir);
     
+    if(!exists(p)) {
+      return;
+    }
     boost::filesystem::directory_iterator end_itr; 
     for ( boost::filesystem::directory_iterator itr( p );
           itr != end_itr;
